/*
	ProWesS, PROGS Windowing System

	PWWindowFit

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 10 February 1995
	20.03.1996 some internal structures are now in pixel coordinates
	24.03.1996 integrated PWSetParentOrigin and PWSetHitCoordinates
	16.04.1997 set pointer position in window, and autoscale object
*/

#include <win.h>

#include "private.h"
#include "PWHandler.h"

/* forward declarations */
static void PWWindowFitRegion(PWRegion *region, pt xinc, pt yinc, int frac,
							  pix pxorg, pix pyorg);
static void PWSetHitCoordinates(PWRegion *region, pix pxorg, pix pyorg);

Error PWWindowFit(PWSystem *system, pt xinc, pt yinc)
{
	PWRegion *region;
	int fraction=0;

	/* set default pointer position for window & clear autoscale object */
	system->win_ptrx=system->curr_xsiz/2;
	system->win_ptry=system->curr_ysiz/2;
	system->autosize=NULL;

	/* determine the total scale factors for the regions */
	for (region=system->regions; region; region=region->next)
		fraction+=region->scale;

	/* actually scale the regions */
	/* AND make sure all coordinates are correct -- parent + hit area */
	for (region=system->regions; region; region=region->next)
		PWWindowFitRegion(region,xinc,yinc,fraction,0,0);

	return ERR_OK;
}

void PWWindowFitRegion(PWRegion *region, pt _xinc, pt _yinc, int frac,
					   pix pxorg, pix pyorg)
{
	PWRegion *child;
	pt siz;
	int fraction=0;
	pt xinc=0, yinc=0;

	if (region->scale)
	{
		if (region->level&1)
			xinc=_xinc*region->scale/frac;
		else
			yinc=_yinc*region->scale/frac;
	}

	if (region->windowfit)
	{
		if (region->level&1)
			yinc=_yinc;
		else
			xinc=_xinc;
	}
	region->tot_xsiz+=xinc;
	region->tot_ysiz+=yinc;

	/* adjust coordinates for following regions on this level */
	if (region->level&1)
		for (child=region->next; child; child=child->next)
			child->tot_xpos+=xinc;
	else
		for (child=region->next; child; child=child->next)
			child->tot_ypos+=yinc;

	/* scale this region by amount specified */
	PWSetHitCoordinates(region,pxorg,pyorg);
	if (region->owner && region->owner->type->PWScale)
		region->owner->type->PWScale(region->owner,xinc,yinc);

	/* determine the total scale factors for the children */
	for (child=region->children; child; child=child->next)
		fraction+=child->scale;

	/* scale the children */
	if (region->level&1)
	{
		siz=region->tot_xsiz-short2pt(region->hit_off_lft+region->hit_off_rgt);
		for (child=region->children; child; child=child->next)
			PWWindowFitRegion(child,siz-child->tot_xsiz,yinc,fraction,
							  region->hit_xorg,region->hit_yorg);
	}
	else
	{
		siz=region->tot_ysiz-short2pt(region->hit_off_top+region->hit_off_bot);
		for (child=region->children; child; child=child->next)
			PWWindowFitRegion(child,xinc,siz-child->tot_ysiz,fraction,
							  region->hit_xorg,region->hit_yorg);
	}
}

static void PWSetHitCoordinates(PWRegion *region, pix pxorg, pix pyorg)
{
	int xpos=region->tot_xpos;
	int ypos=region->tot_ypos;
	PWSystem *system=region->system;

	region->hit_xorg=pt2rshort(xpos)				 +pxorg+region->hit_off_lft;
	region->hit_yorg=pt2rshort(ypos)				 +pyorg+region->hit_off_top;
	region->hit_xend=pt2rshort(xpos+region->tot_xsiz)+pxorg-region->hit_off_rgt;
	region->hit_yend=pt2rshort(ypos+region->tot_ysiz)+pyorg-region->hit_off_bot;

	if (region->owner)
	{
		if (region->owner->status&OSTATUS_POINTER)
		{
			system->win_ptrx=(region->hit_xorg+region->hit_xend)/2;
			system->win_ptry=(region->hit_yorg+region->hit_yend)/2;
		}
		if (region->owner->status&OSTATUS_AUTOSIZE)
			system->autosize=region->owner;
	}
}
