/*
	ProWesS, PROGS Windowing System

	Separator line object "SEPA"
	Container objects "CONT" and "KEEP"

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 17 February 1995
	22.03.1996 adjusted for internal values in pixels
*/

#include <mem.h>

#include "PWHandler.h"

/* global statusses */

#define p100 short2pt(100)
pix xwidth=1, ywidth=1; pt width;	/* width nominal and in each direction */
ColourRGB colour = { p100, 0, 0 }; char Kcolour=FALSE;

/* object definition */
typedef struct {
	PWBaseObject;					/* common start for all objects */
} Object;


/* object handling routines */

Error CONTCreate(PWObject owner, PWObject *ret, PWEvent event, int *taglist)
{
	LinkPROforma();

	event.region->hit_off_lft=event.region->hit_off_rgt=xwidth;
	event.region->hit_off_top=event.region->hit_off_bot=ywidth;
	event.region->composite=TRUE;

	/* allocate specific object - which should be cleared (automatic) */
	return PWAllocate(event.region->system,sizeof(Object),ret);

	/* no events to react to */
}

Error KEEPCreate(PWObject owner, PWObject *ret, PWEvent event, int *taglist)
{
	event.region->composite=TRUE;
	/* allocate specific object - which should be cleared (automatic) */
	return PWAllocate(event.region->system,sizeof(Object),ret);

	/* no events to react to */
}

Error SEPACreate(PWObject owner, PWObject *ret, PWEvent event, int *taglist)
{
	/* no events to react to */
	event.region->scale=0;

	/* allocate specific object - which should be cleared (automatic) */
	return PWAllocate(event.region->system,sizeof(Object),ret);
}

Error Remove(PWObject object)
{
	return PWRelease(object,object);
}

Error CONTDraw(PWObject obj)
{
	PWRegion *reg=obj->event.region;
	Gstate gstate=reg->system->gstate;
	pt xsiz=ypix2PF(reg->hit_xend-reg->hit_xorg);
	pt ysiz=ypix2PF(reg->hit_yend-reg->hit_yorg);
	pt xorg=xpix2PF(reg->hit_xorg);
	pt yorg=ypix2PF(reg->hit_yorg);
	pt x_width=xpix2PF(xwidth)/2;
	pt y_width=ypix2PF(ywidth)/2;

	PFPathStroked(gstate);
	PFColourRGB(gstate, &colour);

	PFLineWidth(gstate, x_width*2);
	PFMoveTo(gstate, xorg	  -x_width, yorg-y_width);
	PFLineR(gstate, 0, ysiz+y_width*2);
	PFMoveTo(gstate, xorg+xsiz+x_width, yorg-y_width);
	PFLineR(gstate, 0, ysiz+y_width*2);
	PFPathDraw(gstate);

	PFLineWidth(gstate, y_width*2);
	PFMoveTo(gstate, xorg-x_width, yorg 	-y_width);
	PFLineR(gstate, xsiz+x_width*2, 0);
	PFMoveTo(gstate, xorg-x_width, yorg+ysiz+y_width);
	PFLineR(gstate, xsiz+x_width*2, 0);
	PFPathDraw(gstate);

	return ERR_OK;
}

Error SEPADraw(PWObject object)
{
	Object *obj=(Object *)object;
	PWRegion *reg=obj->event.region;
	Gstate gstate=reg->system->gstate;
	pix xorg=reg->hit_xorg;
	pix yorg=reg->hit_yorg;

	if (!gstate) return ERR_OK;

	PFPWindowSubSet(gstate,reg->hit_xend-xorg,reg->hit_yend-yorg,xorg,yorg);
	PFPaperColourRGB(gstate, &colour);
	PFPageClear(gstate);
	return PFWindowSubRestore(gstate);
}

void cwidth(void *data)
{
	xwidth=PF2xpix(width); if (xwidth<1) xwidth=1;
	ywidth=PF2ypix(width); if (ywidth<1) ywidth=1;
}

Defines defines[]={
{ "COLOUR",     PW_DEFINE_COLOUR,   NULL,   &colour,    &Kcolour },
{ "THICKNESS",  PW_DEFINE_PT,       cwidth, &width,     NULL },
{ 0 }
};

Error Define(char *name, char *value)
{
	return PWHandleDefine("SEPARATOR-",defines,name,value);
}

Error SEPADetermineSize(PWObject object)
{
	PWRegion *region=object->event.region;

	if (!Kcolour) PWQuery(object,PW_DEFAULT_MIDDLEGROUND,&colour);

	region->hit_off_lft=region->hit_off_rgt=
	region->hit_off_top=region->hit_off_bot=0;

	region->tot_xsiz=short2pt(xwidth);
	region->tot_ysiz=short2pt(ywidth);
	return ERR_OK;
}

Error GetColour(PWObject object)
{
	if (!Kcolour) PWQuery(object,PW_DEFAULT_MIDDLEGROUND,&colour);
	return ERR_OK;
}

PWTypeDef keep={
	PW_TYPEDEF_FLAG, PW_TYPE_DIRECTION, PW_TYPE_EVENT_REGION,
	&KEEPCreate, NULL, NULL, &Remove, NULL, NULL, NULL, NULL,
	NULL, GetColour, NULL
};

PWTypeDef cont={
	(int)&keep, PW_TYPE_CONTAINER, PW_TYPE_EVENT_REGION,
	&CONTCreate, NULL, NULL, &Remove, NULL, NULL, NULL, NULL,
	&CONTDraw, GetColour, NULL
};

PWTypeDef init={
	(int)&cont, PW_TYPE_SEPARATOR, PW_TYPE_EVENT_REGION,
	&SEPACreate, NULL, NULL, &Remove, NULL, NULL, NULL, &Define,
	&SEPADraw, &SEPADetermineSize, NULL
};
